#!/usr/bin/python
# -*- coding: utf-8 -*-

def get_places(username, keyword, lat, lng, db):
    places = []

    return places

def get_place_data(username, place_id, db):
    data = {}

    return data