#!/usr/bin/python
# -*- coding: utf-8 -*-

CONTEXT_SET_MODES = "CONTEXT_SET_MODES"
CONTEXT_GET_SENSOR_DATA = "CONTEXT_GET_SENSOR_DATA"
CONTEXT_SET_HEATING = "CONTEXT_SET_HEATING"
CONTEXT_SET_THERMOSTAT = "CONTEXT_SET_THERMOSTAT"
CONTEXT_CONTROL_BLINDS = "CONTEXT_CONTROL_BLINDS"
CONTEXT_GET_WEATHER = "CONTEXT_GET_WEATHER"
CONTEXT_ADD_NUTRITION = "CONTEXT_ADD_NUTRITION"
CONTEXT_TV_PROGRAMME = "CONTEXT_TV_PROGRAMME"
CONTEXT_WOL = "CONTEXT_WOL"
CONTEXT_CALCULATOR = "CONTEXT_CALCULATOR"